<?php
/**
 *------------------------------------------------------------------------------
 *  iCagenda v3 by Jooml!C - Events Management Extension for Joomla! 2.5 / 3.x
 *------------------------------------------------------------------------------
 * @package     com_icagenda
 * @copyright   Copyright (c)2012-2016 Cyril Rezé, Jooml!C - All rights reserved
 *
 * @license     GNU General Public License version 3 or later; see LICENSE.txt
 * @author      Cyril Rezé (Lyr!C)
 * @link        http://www.joomlic.com
 *
 * @version     3.6.4 2016-10-06
 * @since       3.6.0
 *------------------------------------------------------------------------------
*/

// No direct access to this file
defined('_JEXEC') or die();

if (version_compare(JVERSION, '3.0', 'lt'))
{
	jimport('joomla.application.component.modelitem');
	jimport('joomla.registry.registry');
}

use Joomla\Registry\Registry;


/**
 * iCagenda Component Event Model
 *
 */
class iCagendaModelEvent extends JModelItem
{
	/**
	 * Model context string.
	 *
	 * @var        string
	 */
	protected $_context = 'com_icagenda.event';

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @since	3.6.0
	 *
	 * @return void
	 */
	protected function populateState()
	{
		$app = JFactory::getApplication('site');

		// Load state from the request.
		$pk = $app->input->getInt('id');
		$this->setState('event.id', $pk);

		$offset = $app->input->getUInt('limitstart');
		$this->setState('list.offset', $offset);

		// Load the parameters.
		$params = $app->getParams();
		$this->setState('params', $params);

		// TODO: Tune these values based on other permissions.
		$user = JFactory::getUser();

		if ((!$user->authorise('core.edit.state', 'com_icagenda')) && (!$user->authorise('core.edit', 'com_icagenda')))
		{
			$this->setState('filter.published', 1);
			$this->setState('filter.archived', 2);
		}

		$this->setState('filter.language', JLanguageMultilang::isEnabled());
	}

	/**
	 * Method to get event data.
	 *
	 * @param   integer  $pk  The id of the event.
	 *
	 * @return  mixed  Menu item data object on success, false on failure.
	 *
	 * @since	3.6.0
	 */
	public function getItem($pk = null)
	{
		$app	= JFactory::getApplication();
		$user	= JFactory::getUser();

		$jinput	= $app->input;

		$pk = (!empty($pk)) ? $pk : (int) $this->getState('event.id');

		if ($this->_item === null)
		{
			$this->_item = array();
		}

		if (!isset($this->_item[$pk]))
		{
			try
			{
				$db = $this->getDbo();
				$query = $db->getQuery(true)
					->select(
						$this->getState(
							'item.select', 'e.*, ' .
							'e.name AS contact_name, e.email AS contact_email'
						)
					);
				$query->from('#__icagenda_events AS e');
				$query->where('e.state = 1');

				// Join over the category
//				$query->select('c.title AS cat_title, c.color AS cat_color')
//					->join('LEFT', '#__icagenda_category AS c on c.id = e.catid');
				$query->select('c.id AS cat_id, c.title AS cat_title, c.color AS cat_color, c.desc AS cat_desc');
				$query->join('LEFT', '#__icagenda_category AS c ON c.id = e.catid');
				$query->where('c.state = 1');

				// Filter by language
				if ($this->getState('filter.language'))
				{
					$query->where('e.language in (' . $db->quote(JFactory::getLanguage()->getTag()) . ',' . $db->quote('*') . ')');
				}

				$query->where('e.id = ' . (int) $pk);

				// Features - extract the number of displayable icons per event
				$query->select('feat.count AS features');
				$sub_query = $db->getQuery(true);
				$sub_query->select('fx.event_id, COUNT(*) AS count');
				$sub_query->from('`#__icagenda_feature_xref` AS fx');
				$sub_query->innerJoin("`#__icagenda_feature` AS f ON fx.feature_id=f.id AND f.state=1 AND f.icon<>'-1'");
				$sub_query->group('fx.event_id');
				$query->leftJoin('(' . (string) $sub_query . ') AS feat ON e.id=feat.event_id');

				// Registrations total
				$query->select('r.count AS registered');
				$sub_query = $db->getQuery(true);
				$sub_query->select('r.eventid, sum(r.people) AS count');
				$sub_query->from('`#__icagenda_registration` AS r');

				$get_date = $jinput->get('date', '');

				if ($get_date)
				{
					$ex = explode('-', $get_date);

					if (strlen(iCDate::dateToNumeric($get_date)) != '12')
					{
						$event_url	= JUri::getInstance()->toString();
						$cleanurl	= preg_replace('/&date=[^&]*/', '', $event_url);
						$cleanurl	= preg_replace('/\?date=[^\?]*/', '', $cleanurl);

						// redirect and remove date var, if not correctly set
//						$app->redirect($cleanurl , JText::_( 'COM_ICAGENDA_ERROR_URL_DATE_NOT_FOUND' ));
						$app->redirect($cleanurl);

						return false;
					}

					if (count($ex) == 5)
					{
						$dateday = $ex['0'] . '-' . $ex['1'] . '-' . $ex['2'] . ' ' . $ex['3'] . ':' . $ex['4'] . ':00';

						$sub_query->where('r.state = 1 AND (r.date = ' . $db->q($dateday) . ' OR r.period = 1 OR (r.period <> 1 AND r.date = ""))');
					}
				}
				else
				{
					$sub_query->where('r.state = 1');
				}

				$sub_query->group('r.eventid');
				$query->leftJoin('(' . (string) $sub_query . ') AS r ON e.id = r.eventid');

				// Filter by published state.
//				$published = $this->getState('filter.published');
//				$archived = $this->getState('filter.archived');

//				if (is_numeric($published))
//				{
//					$query->where('(a.state = ' . (int) $published . ' OR a.state =' . (int) $archived . ')');
//				}

				// Filter by language
				if ($this->getState('filter.language'))
				{
					$query->where('e.language in (' . $db->q(JFactory::getLanguage()->getTag()) . ',' . $db->q('*') . ')');
				}

				$db->setQuery($query);

				$data = $db->loadObject();

				if (empty($data))
				{
					return JError::raiseError(404, JText::_('COM_ICAGENDA_ERROR_EVENT_NOT_FOUND'));
				}

				// iCagenda event view variables
				$data->backArrow			= icagendaEvent::backArrow($data);
				$data->map					= icagendaGooglemaps::map($data);
				$data->managerToolBar		= icagendaManager::toolBar($data);
//				$data->maxNbTickets			= icagendaRegistration::maxNbTickets($data);
				$data->totalRegistered		= icagendaRegistration::totalRegistered($data);

				// Add to Cal
				$data->googleCalendar		= icagendaAddtocal::googleCalendar($data);
				$data->windowsliveCalendar	= icagendaAddtocal::windowsliveCalendar($data);
				$data->yahooCalendar		= icagendaAddtocal::yahooCalendar($data);

				// Extract the feature details, if needed
				if (is_null($data->features))
				{
					$data->features = array();
				}
				else
				{
					$db = $this->getDbo();
					$query = $db->getQuery(true);
					$query->select('DISTINCT f.icon, f.icon_alt');
					$query->from('`#__icagenda_feature_xref` AS fx');
					$query->innerJoin("`#__icagenda_feature` AS f ON fx.feature_id=f.id AND f.state=1 AND f.icon<>'-1'");
					$query->where('fx.event_id=' . $data->id);
					$query->order('f.ordering DESC'); // Order descending because the icons are floated right
					$db->setQuery($query);
					$data->features = $db->loadObjectList();
				}

				// Check for published state if filter set.
//				if (((is_numeric($published)) || (is_numeric($archived))) && (($data->state != $published) && ($data->state != $archived)))
//				{
//					return JError::raiseError(404, JText::_('COM_ICAGENDA_ERROR_EVENT_NOT_FOUND'));
//				}

				// Convert parameter fields to objects.
				$registry = (version_compare(JVERSION, '3.0', 'lt')) ? new JRegistry : new Registry;
				$registry->loadString($data->params);

				// Merge Event params to app params
				$data->params = clone $this->getState('params');
				$data->params->merge($registry);

				$this->_item[$pk] = $data;
			}
			catch (Exception $e)
			{
				if ($e->getCode() == 404)
				{
					// Need to go thru the error handler to allow Redirect to work.
					JError::raiseError(404, $e->getMessage());
				}
				else
				{
					$this->setError($e);
					$this->_item[$pk] = false;
				}
			}
		}

		return $this->_item[$pk];
	}

	/**
	 * Increment the hit counter for the event.
	 *
	 * @param   integer  $pk  Optional primary key of the event to increment.
	 *
	 * @return  boolean  True if successful; false otherwise and internal error set.
	 *
	 * @since	3.6.0
	 */
	public function hit($pk = 0)
	{
		$input = JFactory::getApplication()->input;
		$hitcount = $input->getInt('hitcount', 1);

		if ($hitcount)
		{
			// Initialise variables.
			$pk = (!empty($pk)) ? $pk : (int) $this->getState('event.id');
			$db = $this->getDbo();

			$db->setQuery(
					'UPDATE #__icagenda_events' .
					' SET hits = hits + 1' .
					' WHERE id = ' . (int) $pk
			);

			if ( ! $db->query())
			{
				$this->setError($db->getErrorMsg());

				return false;
            }
        }

        return true;
	}
}
